/**
 * FifteenPuzzle.js
 * ----------------
 * This program animates the Fifteen Puzzle.
 */

"use strict";

/* Constants */

const SQUARE_SIZE = 60;
const INSET = 2;
const TILE_SIZE = SQUARE_SIZE - 2 * INSET;
const GWINDOW_WIDTH = 4 * SQUARE_SIZE;
const GWINDOW_HEIGHT = GWINDOW_WIDTH;
const TILE_FONT = "18px 'Times New Roman'";

/**
 * Function: FifteenPuzzle
 * -----------------------
 * Defines the factory function that manages the entire
 * simulation.
 */
function FifteenPuzzle() {
	let gw = GWindow(GWINDOW_WIDTH, GWINDOW_HEIGHT);
	initFifteenPuzzle(gw);
	let clickAction = function (e) {
		let tile = gw.getElementAt(e.getX(), e.getY());
		if (tile === null) return;
		if (tryToMove(gw, tile, SQUARE_SIZE, 0)) return;
		if (tryToMove(gw, tile, -SQUARE_SIZE, 0)) return;
		if (tryToMove(gw, tile, 0, SQUARE_SIZE)) return;
		tryToMove(gw, tile, 0, -SQUARE_SIZE);
	};
	gw.addEventListener("click", clickAction);
}

/**
 * Function: initFifteenPuzzle
 * ---------------------------
 * Constructs the initial state of the Fifteen Puzzle
 * board by properly implanting fifteen numbered squares
 * into the supplied GWindow.
 */
function initFifteenPuzzle(gw) {
	let x = 0;
	let y = 0;
	for (let number = 1; number < 16; number++) {
		gw.add(createNumberedSquare(number), x, y);
		if (number % 4 === 0) { // yes? time to advance down a row
			x = 0;
			y += SQUARE_SIZE;
		} else { // no? jimmy over to the right
			x += SQUARE_SIZE;
		}
	}
}

/**
 * Function: createNumberedSquare
 * ------------------------------
 * Constructs a GCompound consisting of a square (GRect) with
 * a centered number (GLabel) with a reference point set at
 * the upper left corner of the square.
 */
function createNumberedSquare(number) {
	let tile = GCompound();
	let square = GRect(0, 0, TILE_SIZE, TILE_SIZE);
	square.setFilled(true);
	square.setFillColor("LightGray");
	tile.add(square, INSET, INSET);
	let label = GLabel("" + number);
	label.setFont(TILE_FONT);
	let cx = (SQUARE_SIZE - label.getWidth())/2;
	let cy = (SQUARE_SIZE + label.getAscent())/2;
	tile.add(label, cx, cy);
	return tile;
}

/**
 * Predicate function: tryToMove
 * -----------------------------
 * Uses the upper left corner of the tile provided via the second
 * argument to compute the upper left corner of a neighboring tile.
 * If the neighboring (ulx, uly) corner is outside the bounds of the window,
 * or it's the upper left corner of an actual tile, then the original tile
 * is blocked and can't be moved, and tryToMove expresses failure by returning
 * false.  If (ulx, uly) overlays the empty square, then the original tile
 * is shifted into the void and true is returned instead.
 */
function tryToMove(gw, tile, dx, dy) {
	let ulx = tile.getX() + dx + INSET + 1;
	let uly = tile.getY() + dy + INSET + 1;
	if (ulx < 0 || ulx >= gw.getWidth()) return false;
	if (uly < 0 || uly >= gw.getHeight()) return false;
	if (gw.getElementAt(ulx, uly) !== null) return false; // not really empty then
	tile.move(dx, dy);
	return true;
}
