/**
 * File: ElectionData.js
 * ---------------------
 * Defines the ElectionData class, which knows
 * how to parse the embedded XML of the index.html, and
 * construct an internal representation that supports these
 * two methods, as documented in the problem statement:
 *
 * + getConstituencyNames();
 * + getResults(name);
 */

"use strict";

/**
 * Function: ElectionData
 * ----------------------
 * Defines the factory function that constructs a class with two
 * exposed methods, as defined in the practice exam.  constituencyNames
 * and constituenciesMap are immediately defined where they are so that they
 * are part of the closure accessed by the implementations of getConstituencyNames
 * and getResults.  parseXML is the meat of the implementation, that knows how
 * to process the inner XML that catalogues all of the election data stored
 * in a format that's detailed in the problem statement.
 */
function ElectionData() {
	let constituencyNames = [];
	let constituenciesMap = {};
	parseXML(constituencyNames, constituenciesMap);
	return {
		getConstituencyNames: function() { 
			return constituencyNames;
		},
		getResults: function(name) {
			let results = constituenciesMap[name];
			if (results === undefined) results = [];
			return results;
		}
	};
}

/**
 * Function: parseXML
 * ------------------
 * Accepts the empty constituencyNames array and the
 * empty constituenciesMap map, and populates them
 * with the names of all constituencies and the collection
 * of key/value pairs that represent the relevant
 * constituent-name -> election-results information.
 */
function parseXML(constituencyNames, constituenciesMap) {
	let electionXML = document.getElementById("ElectionData");
	let constituencies = electionXML.getElementsByTagName("constituency");
	for (let i = 0; i < constituencies.length; i++) {
		let constituency = constituencies[i];
		let constituencyName = constituency.getAttribute("name");
		constituencyNames.push(constituencyName);
		constituenciesMap[constituencyName] = [];
		let candidates = constituency.getElementsByTagName("candidate");
		for (let j = 0; j < candidates.length; j++) {
			let candidate = candidates[j];
			let entry = {
				candidate: candidate.getAttribute("name"),
				party: candidate.getAttribute("party"),
				votes: parseInt(candidate.getAttribute("votes"))
			};
			constituenciesMap[constituencyName].push(entry);
		}
	}
}

/**
 * Function: TestElectionData
 * --------------------------
 * Simple test harness to confirm that the ElectionData
 * class methods (and the parseXML helper function) all
 * work as expected.
 */
function TestElectionData() {
   let electionData = ElectionData();
   let totalLabourVote = 0;
   let constituencies = electionData.getConstituencyNames();
   for (let i = 0; i < constituencies.length; i++) {
      let results = electionData.getResults(constituencies[i]);
      for (let j = 0; j < results.length; j++) {
         if (results[j].party === "Labour") {
            totalLabourVote += results[j].votes;
         }
      }
   }
   console.log("Total Labour vote = " + totalLabourVote);
}
